"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
const web3_1 = __importDefault(require("web3"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const origin_backend_client_mocks_1 = require("@energyweb/origin-backend-client-mocks");
const migrate_1 = require("../migrate");
const __1 = require("..");
const Logger_1 = require("../Logger");
describe('Issuer', () => {
    let issuer;
    let registry;
    let conf;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const web3 = new web3_1.default(process.env.WEB3);
    const deployKey = process.env.DEPLOY_KEY;
    const privateKeyDeployment = deployKey.startsWith('0x') ? deployKey : `0x${deployKey}`;
    const accountDeployment = web3.eth.accounts.privateKeyToAccount(privateKeyDeployment).address;
    const deviceOwnerPK = '0xd9bc30dc17023fbb68fe3002e0ff9107b241544fd6d60863081c55e383f1b5a3';
    const accountDeviceOwner = web3.eth.accounts.privateKeyToAccount(deviceOwnerPK).address;
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerAccount = web3.eth.accounts.privateKeyToAccount(issuerPK).address;
    let timestamp = moment_1.default().subtract(10, 'year').unix();
    const setActiveUser = (privateKey) => {
        conf.blockchainProperties.activeUser = {
            address: web3.eth.accounts.privateKeyToAccount(privateKey).address,
            privateKey
        };
    };
    const createRequestIssue = async (conf, isPrivate = false) => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const deviceId = '1';
        return __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf, isPrivate);
    };
    it('migrates Issuer and Registry', async () => {
        registry = await migrate_1.migrateRegistry(web3, privateKeyDeployment);
        issuer = await migrate_1.migrateIssuer(web3, privateKeyDeployment, registry.web3Contract.options.address);
        const version = await issuer.version();
        chai_1.assert.equal(version, 'v0.1');
        const registryAddress = await issuer.getRegistryAddress();
        chai_1.assert.equal(registryAddress, registry.web3Contract.options.address);
        conf = {
            blockchainProperties: {
                activeUser: {
                    address: accountDeployment,
                    privateKey: privateKeyDeployment
                },
                issuerLogicInstance: issuer,
                web3
            },
            offChainDataSource: new origin_backend_client_mocks_1.OffChainDataSourceMock(),
            logger: Logger_1.logger
        };
    });
    it('user correctly requests issuance', async () => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const deviceId = '1';
        const requestIssue = await __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf);
        chai_1.assert.isAbove(Number(requestIssue.id), -1);
        chai_1.assert.deepOwnInclude(requestIssue, {
            initialized: true,
            deviceId,
            owner: accountDeviceOwner,
            fromTime,
            toTime,
            approved: false,
            isPrivate: false
        });
    });
    it('issuer correctly approves issuance', async () => {
        let requestIssue = await createRequestIssue(conf);
        setActiveUser(issuerPK);
        const volume = 1000;
        const certificateId = await requestIssue.approve(accountDeviceOwner, volume);
        requestIssue = await requestIssue.sync();
        chai_1.assert.isTrue(requestIssue.approved);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance, volume);
    });
    it('issuer revokes a certificate', async () => {
        let requestIssue = await createRequestIssue(conf);
        setActiveUser(issuerPK);
        const volume = 1000;
        const certificateId = await requestIssue.approve(accountDeviceOwner, volume);
        requestIssue = await requestIssue.sync();
        chai_1.assert.isTrue(requestIssue.approved);
        chai_1.assert.isFalse(requestIssue.revoked);
        await requestIssue.revoke();
        requestIssue = await requestIssue.sync();
        chai_1.assert.isTrue(requestIssue.revoked);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance, volume);
    });
    it('should fail to request 2 certificates with the same generation period', async () => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const deviceId = '1';
        await __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf);
        let failed = false;
        try {
            await __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf);
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('should request the same certificate after revoking one', async () => {
        setActiveUser(deviceOwnerPK);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const deviceId = '1';
        let requestIssue = await __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf);
        setActiveUser(issuerPK);
        const volume = 1000;
        await requestIssue.approve(accountDeviceOwner, volume);
        requestIssue = await requestIssue.sync();
        await requestIssue.revoke();
        requestIssue = await requestIssue.sync();
        const newRequestIssue = await __1.RequestIssue.createRequestIssue(fromTime, toTime, deviceId, conf);
        chai_1.assert.exists(newRequestIssue);
    });
    it('user correctly requests private issuance', async () => {
        const requestIssue = await createRequestIssue(conf, true);
        chai_1.assert.isAbove(Number(requestIssue.id), -1);
        chai_1.assert.deepOwnInclude(requestIssue, {
            initialized: true,
            deviceId: '1',
            owner: accountDeviceOwner,
            approved: false,
            isPrivate: true
        });
    });
    it('issuer correctly approves private issuance', async () => {
        let requestIssue = await createRequestIssue(conf, true);
        setActiveUser(issuerPK);
        const volume = 1000;
        const certificateId = await requestIssue.approve(accountDeviceOwner, volume);
        requestIssue = await requestIssue.sync();
        chai_1.assert.isTrue(requestIssue.approved);
        const deviceOwnerBalance = await registry.balanceOf(accountDeviceOwner, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance, 0);
    });
});
//# sourceMappingURL=Issuer.test.js.map